import unittest
import json
import os
from botocore.exceptions import WaiterError
from .client import SQSClient


class TestSQSClientIntegration(unittest.TestCase):

    @classmethod
    def setUpClass(cls) -> None:
        cls.region_name = os.environ.get("AWS_SQS_QUEUE_REGION")
        cls.aws_access_key_id = os.environ.get("AWS_SQS_QUEUE_ACCESS_KEY_ID")
        cls.aws_secret_access_key = os.environ.get("AWS_SQS_QUEUE_SECRET_ACCESS_KEY")
        cls.queue_url = os.environ.get("AWS_SQS_QUEUE_TEST_URL")

        cls.client = SQSClient(cls.region_name, cls.aws_access_key_id, cls.aws_secret_access_key, cls.queue_url)

    def setUp(self) -> None:
        self.client.purge()

    def tearDown(self) -> None:
        self.client.purge()

    def test_send_and_read_message(self):
        # arrange
        event_type = "LPViewEvent"
        component = "web_layer"
        body = {
            "message": "test_message",
            "cls": "LPViewEvent",
            "p": {
                "account_id": "601eeee58ff0e51d829455cc",
                "c": 12423232,
                "pid": 635424536546,
            },
        }

        try:
            # act
            self.client.send_message(body, event_type, component)
            messages = self.client.read_events_with_retry(max_retry_attempts=3, max_batch=10)

            # assert
            self.assertEqual(len(messages), 1)
            self.assertEqual(messages[0]['Body'], json.dumps(body))
            self.assertEqual(messages[0]['MessageAttributes']['event_type']['StringValue'], event_type)
            self.assertEqual(messages[0]['MessageAttributes']['component']['StringValue'],
                             component)  # Check the component attribute

        except WaiterError as e:
            self.fail("Error reading message from the queue: {}".format(str(e)))



from setuptools import setup, find_packages

setup(
    name='simple_sqs_client',
    version='0.9.5',
    author='Aleksander Laurowski',
    author_email='aleksander.laurowski1991@gmail.com',
    description='A very simple package for interacting with SQS (Simple Queue Service) in AWS',
    packages=find_packages(),
    install_requires=['boto3'],
)



from .client import SQSClient


class SQSClientBuilder:
    """
    A builder class for creating an SQSClient with different queue and AWS credentials.
    """

    def __init__(self):
        self.region_name = None
        self.aws_access_key_id = None
        self.aws_secret_access_key = None
        self.queue_url = None

    def with_region(self, region_name: str) -> 'SQSClientBuilder':
        """
        Sets the AWS region name.

        Args:
            region_name (str): The AWS region name.

        Returns:
            SQSClientBuilder: The updated builder instance.
        """
        self.region_name = region_name
        return self

    def with_aws_credentials(self, aws_access_key_id: str, aws_secret_access_key: str) -> 'SQSClientBuilder':
        """
        Sets the AWS access key ID and secret access key.

        Args:
            aws_access_key_id (str): The AWS access key ID.
            aws_secret_access_key (str): The AWS secret access key.

        Returns:
            SQSClientBuilder: The updated builder instance.
        """
        self.aws_access_key_id = aws_access_key_id
        self.aws_secret_access_key = aws_secret_access_key
        return self

    def with_queue_url(self, queue_url: str) -> 'SQSClientBuilder':
        """
        Sets the SQS queue URL.

        Args:
            queue_url (str): The SQS queue URL.

        Returns:
            SQSClientBuilder: The updated builder instance.
        """
        self.queue_url = queue_url
        return self

    def build(self) -> 'SQSClient':
        """
        Builds an instance of SQSClient with the provided configuration.

        Returns:
            SQSClient: An instance of SQSClient.
        """
        missing_fields = []

        if not self.region_name:
            missing_fields.append('region_name')
        if not self.aws_access_key_id:
            missing_fields.append('aws_access_key_id')
        if not self.aws_secret_access_key:
            missing_fields.append('aws_secret_access_key')
        if not self.queue_url:
            missing_fields.append('queue_url')

        if missing_fields:
            raise ValueError(f"Missing required SQS connection parameters: {', '.join(missing_fields)}")

        client = SQSClient(self.region_name, self.aws_access_key_id, self.aws_secret_access_key, self.queue_url)
        return client




import boto3
from botocore.exceptions import WaiterError, HTTPClientError
import logging


class SQSClient:
    """
    A client for interacting with an SQS (Simple Queue Service) in AWS.
    """

    _instances = []

    def __new__(cls, *args, **kwargs) -> 'SQSClient':
        """
               Creates and returns a new instance of the SQSClient class or returns an existing instance with the same parameters.

               Args:
                   *args: Positional arguments passed during instance creation.
                   **kwargs: Keyword arguments passed during instance creation.

               Returns:
                   SQSClient: Newly created instance of the SQSClient class or an existing instance with the same parameters.

               """
        existing_instance = cls._get_existing_instance(*args, **kwargs)
        if existing_instance:
            return existing_instance
        else:
            new_instance = super().__new__(cls)
            cls._instances.append(new_instance)
            return new_instance

    def __init__(self, region_name: str, aws_access_key_id: str, aws_secret_access_key: str,
                 queue_url: str) -> 'SQSClient':
        """
        Initializes the SQS client with the specified AWS credentials and region.

        Args:
            region_name (str): The AWS region name.
            aws_access_key_id (str): The AWS access key ID.
            aws_secret_access_key (str): The AWS secret access key.
            queue_url (str): The AWS SQS queue name
        """
        self.region_name = region_name
        self.aws_access_key_id = aws_access_key_id
        self.aws_secret_access_key = aws_secret_access_key
        self.queue_url = queue_url

        self.logger = logging.getLogger(__name__)

        self._init_client()

    @classmethod
    def _get_existing_instance(
            cls, region_name: str,
            aws_access_key_id: str,
            aws_secret_access_key: str,
            queue_url: str) -> 'SQSClient':
        """
            Checks if an instance of the SQSClient class already exists with the same parameters.

            Args:
                region_name (str): AWS region name.
                aws_access_key_id (str): AWS access key ID.
                aws_secret_access_key (str): AWS secret access key.
                queue_url (str): SQS queue URL.

            Returns:
                SQSClient or None: Existing instance of SQSClient with the same parameters, or None if it doesn't exist.

            """
        for instance in cls._instances:
            if instance.region_name == region_name \
                    and instance.aws_access_key_id == aws_access_key_id \
                    and instance.aws_secret_access_key == aws_secret_access_key \
                    and instance.queue_url == queue_url:
                return instance
        return None

    def __enter__(self):
        self._init_client()
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        if self.sqs is not None:
            self.sqs.close()

    def _init_client(self):
        """
        Initializes the SQS client using the specified AWS credentials and region.
        """
        self.sqs = boto3.client('sqs', region_name=self.region_name,
                                aws_access_key_id=self.aws_access_key_id,
                                aws_secret_access_key=self.aws_secret_access_key)

    def send_message(self, body: str) -> dict:
        """
        Sends a message to the SQS queue.

        Args:
            body (str): The body of the message as a dictionary dumped to JSON string.
        Returns:
            dict: response from AWS SQS
        Raises:
            Exception: If an error occurs while sending the message.
        """
        try:
            response = self.sqs.send_message(
                QueueUrl=self.queue_url,
                MessageBody=body
            )
            self.logger.debug("Message sent: ID = %s", response["MessageId"])
            return response
        except Exception as e:
            self.logger.error("Failed to send message: %s", str(e))
            raise

    def read_events_with_retry(self, max_retry_attempts: int = 3, max_batch: int = 10,
                               visibility_timeout: int = 60, wait_time_seconds: int = 10) -> list:
        """
        Reads events from the SQS queue with retry.

        Args:
            wait_time_seconds (int: The amount of seconds client will be polling messages from the queue
            visibility_timeout (int): The amount of seconds,a messages stays inside the queue before getting removed
            max_retry_attempts (int): The maximum number of retry attempts.
            max_batch (int): The maximum number of messages to retrieve in each batch.

        Returns:
            list: A list of messages retrieved from the queue.

        Raises:
            HTTPClientError: If an error occurs while reading from the queue.
        """
        retry_attempt = 1

        while retry_attempt <= max_retry_attempts:
            try:
                response = self.sqs.receive_message(
                    QueueUrl=self.queue_url,
                    MaxNumberOfMessages=max_batch,
                    MessageAttributeNames=['All'],
                    AttributeNames=['All'],
                    VisibilityTimeout=visibility_timeout,
                    WaitTimeSeconds=wait_time_seconds,
                )

                messages = response.get('Messages', [])

                return messages

            except WaiterError as e:
                self.logger.debug("WaiterError: %s", str(e))
                self.logger.debug("Retry attempt: %s", retry_attempt)
                self._init_client()
                if retry_attempt == max_retry_attempts:
                    self.logger.error("Max retry attempts reached. Exiting.")
                    raise HTTPClientError

                retry_attempt += 1

    def delete_event_from_queue(self, event: dict) -> None:
        """
        Deletes an event from the SQS queue.

        Args:
            event (dict): The event dictionary.

        Returns:
            None
        """
        self.sqs.delete_message(
            QueueUrl=self.queue_url,
            ReceiptHandle=event['ReceiptHandle']
        )

    def purge(self) -> None:
        """
            Purges all messages from an SQS queue.
            Used for test purposes

            Returns:
                None
            """
        try:
            self.sqs.purge_queue(QueueUrl=self.queue_url)
            print("All messages have been purged from the queue.")
        except Exception as e:
            print(f"Failed to purge the queue: {str(e)}")
